@@ -1,5 +1,5 @@
 /*
- * "$Id: png.c,v 1.108.2.2 2004/05/27 15:24:52 neo Exp $"
+ * "$Id: png.c,v 1.118 2004/11/23 14:28:43 mitch Exp $"
  *
  *   Portable Network Graphics (PNG) plug-in for The GIMP -- an image
  *   manipulation program
@@ -109,40 +109,43 @@ PngSaveGui;
  * Local functions...
  */
 
-static void query (void);
-static void run   (const gchar      *name,
-                   gint              nparams,
-                   const GimpParam  *param,
-                   gint             *nreturn_vals,
-                   GimpParam       **return_vals);
+static void      query                     (void);
+static void      run                       (const gchar      *name,
+                                            gint              nparams,
+                                            const GimpParam  *param,
+                                            gint             *nreturn_vals,
+                                            GimpParam       **return_vals);
 
-static gint32 load_image (const gchar *filename,
-                          gboolean     interactive);
-static gint   save_image (const gchar *filename,
-                          gint32       image_ID,
-                          gint32       drawable_ID,
-                          gint32       orig_image_ID);
+static gint32    load_image                (const gchar      *filename,
+                                            gboolean          interactive);
+static gint      save_image                (const gchar      *filename,
+                                            gint32            image_ID,
+                                            gint32            drawable_ID,
+                                            gint32            orig_image_ID);
 
-static void respin_cmap (png_structp   pp,
-                         png_infop     info,
-                         guchar       *remap,
-                         gint32        image_ID,
-                         GimpDrawable *drawable);
+static void      respin_cmap               (png_structp       pp,
+                                            png_infop         info,
+                                            guchar           *remap,
+                                            gint32            image_ID,
+                                            GimpDrawable     *drawable);
 
-static gint save_dialog      (gint32     image_ID,
-                              gboolean   alpha);
+static gboolean  save_dialog               (gint32            image_ID,
+                                            gboolean          alpha);
 
-static void save_dialog_response (GtkWidget     *widget,
-                                  gint           response_id,
-                                  gpointer       data);
+static void      save_dialog_response      (GtkWidget        *widget,
+                                            gint              response_id,
+                                            gpointer          data);
 
-static int find_unused_ia_colour (guchar *pixels,
-                                  gint    numpixels,
-                                  gint   *colors);
+static gboolean  ia_has_transparent_pixels (guchar           *pixels,
+                                            gint              numpixels);
 
-static gboolean load_defaults     (void);
-static void     save_defaults     (void);
-static void     load_gui_defaults (PngSaveGui *pg);
+static int       find_unused_ia_colour     (guchar           *pixels,
+                                            gint              numpixels,
+                                            gint             *colors);
+
+static gboolean  load_defaults             (void);
+static void      save_defaults             (void);
+static void      load_gui_defaults         (PngSaveGui       *pg);
 
 /*
  * Globals...
@@ -243,25 +246,31 @@ query (void)
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>",
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>, Nick Lamb <njl195@zepler.org.uk>",
                           PLUG_IN_VERSION,
-                          "<Load>/PNG",
+                          N_("PNG image"),
                           NULL,
                           GIMP_PLUGIN,
                           G_N_ELEMENTS (load_args),
                           G_N_ELEMENTS (load_return_vals),
                           load_args, load_return_vals);
 
+  gimp_register_file_handler_mime ("file_png_load", "image/png");
+  gimp_register_magic_load_handler ("file_png_load",
+                                    "png", "", "0,string,\211PNG\r\n\032\n");
+
   gimp_install_procedure ("file_png_save",
                           "Saves files in PNG file format",
                           "This plug-in saves Portable Network Graphics (PNG) files.",
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>",
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>, Nick Lamb <njl195@zepler.org.uk>",
                           PLUG_IN_VERSION,
-                          NULL,
+                          N_("PNG image"),
                           "RGB*,GRAY*,INDEXED*",
                           GIMP_PLUGIN,
                           G_N_ELEMENTS (save_args), 0,
                           save_args, NULL);
 
+  gimp_register_file_handler_mime ("file_png_save", "image/png");
+
   gimp_install_procedure ("file_png_save2",
                           "Saves files in PNG file format",
                           "This plug-in saves Portable Network Graphics (PNG) files. "
@@ -269,29 +278,27 @@ query (void)
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>",
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>, Nick Lamb <njl195@zepler.org.uk>",
                           PLUG_IN_VERSION,
-                          NULL,
+                          N_("PNG image"),
                           "RGB*,GRAY*,INDEXED*",
                           GIMP_PLUGIN,
                           G_N_ELEMENTS (save_args2), 0,
                           save_args2, NULL);
 
-  /* We only register a save menu path for this one, since it is the only one
-   * registered as a save handler below.
-   */
+  gimp_register_file_handler_mime ("file_png_save2", "image/png");
+
   gimp_install_procedure ("file_png_save_defaults",
                           "Saves files in PNG file format",
                           "This plug-in saves Portable Network Graphics (PNG) files, using the default settings stored as a parasite.",
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>",
                           "Michael Sweet <mike@easysw.com>, Daniel Skarda <0rfelyus@atrey.karlin.mff.cuni.cz>, Nick Lamb <njl195@zepler.org.uk>",
                           PLUG_IN_VERSION,
-                          "<Save>/PNG",
+                          N_("PNG image"),
                           "RGB*,GRAY*,INDEXED*",
                           GIMP_PLUGIN,
                           G_N_ELEMENTS (save_args_defaults), 0,
                           save_args_defaults, NULL);
 
-  gimp_register_magic_load_handler ("file_png_load",
-                                    "png", "", "0,string,\211PNG\r\n\032\n");
+  gimp_register_file_handler_mime ("file_png_save_defaults", "image/png");
   gimp_register_save_handler ("file_png_save_defaults", "png", "");
 
   gimp_install_procedure ("file_png_get_defaults",
@@ -900,17 +907,17 @@ load_image (const gchar *filename,
           /*  keep at least one entry  */
           empty = MIN (empty, info->num_palette - 1);
 
-          gimp_image_set_cmap (image, (guchar *) (info->palette + empty),
-                               info->num_palette - empty);
+          gimp_image_set_colormap (image, (guchar *) (info->palette + empty),
+                                   info->num_palette - empty);
         }
       else
         {
-          gimp_image_set_cmap (image, (guchar *) info->palette,
-                               info->num_palette);
+          gimp_image_set_colormap (image, (guchar *) info->palette,
+                                   info->num_palette);
         }
 #else
-      gimp_image_set_cmap (image, (guchar *) info->palette,
-                           info->num_palette);
+      gimp_image_set_colormap (image, (guchar *) info->palette,
+                               info->num_palette);
 #endif /* PNG_LIBPNG_VER > 99 */
 
     }
@@ -1231,7 +1238,7 @@ save_image (const gchar *filename,
       info->color_type = PNG_COLOR_TYPE_PALETTE;
       info->valid |= PNG_INFO_PLTE;
       info->palette =
-        (png_colorp) gimp_image_get_cmap (image_ID, &num_colors);
+        (png_colorp) gimp_image_get_colormap (image_ID, &num_colors);
       info->num_palette = num_colors;
       break;
     case GIMP_INDEXEDA_IMAGE:
@@ -1268,7 +1275,7 @@ save_image (const gchar *filename,
     {
       GimpRGB color;
 
-      gimp_palette_get_background (&color);
+      gimp_context_get_background (&color);
       gimp_rgb_get_uchar (&color, &red, &green, &blue);
 
       background.index = 0;
@@ -1292,7 +1299,7 @@ save_image (const gchar *filename,
       parasite = gimp_image_parasite_find (orig_image_ID, "gamma");
       if (parasite)
 	{
-          gamma = g_ascii_strtod (parasite->data, NULL);
+          gamma = g_ascii_strtod (gimp_parasite_data (parasite), NULL);
 	  gimp_parasite_free (parasite);
 	}
 
@@ -1457,6 +1464,19 @@ save_image (const gchar *filename,
   return (1);
 }
 
+static gboolean
+ia_has_transparent_pixels (guchar *pixels,
+                           gint    numpixels)
+{
+  while (numpixels --)
+    {
+      if (pixels [1] <= 127)
+        return TRUE;
+      pixels += 2;
+    }
+  return FALSE;
+}
+
 static void
 respin_cmap (png_structp pp,
              png_infop info,
@@ -1472,8 +1492,9 @@ respin_cmap (png_structp pp,
   gint          cols, rows;
   GimpPixelRgn  pixel_rgn;
   guchar       *pixels;
+  gint          numpixels;
 
-  before = gimp_image_get_cmap (image_ID, &colors);
+  before = gimp_image_get_colormap (image_ID, &colors);
 
   /*
    * Make sure there is something in the colormap.
@@ -1484,13 +1505,14 @@ respin_cmap (png_structp pp,
       colors = 1;
     }
 
-  cols = drawable->width;
-  rows = drawable->height;
+  cols      = drawable->width;
+  rows      = drawable->height;
+  numpixels = cols * rows;
 
   gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0,
                        drawable->width, drawable->height, FALSE, FALSE);
 
-  pixels = (guchar *) g_malloc (drawable->width * drawable->height * 2);
+  pixels = (guchar *) g_malloc (numpixels * 2);
 
   gimp_pixel_rgn_get_rect (&pixel_rgn, pixels, 0, 0,
                            drawable->width, drawable->height);
@@ -1499,48 +1521,52 @@ respin_cmap (png_structp pp,
   /* Try to find an entry which isn't actually used in the
      image, for a transparency index. */
 
-  transparent = find_unused_ia_colour (pixels,
-                                       drawable->width * drawable->height,
-                                       &colors);
-
 #if PNG_LIBPNG_VER > 99
-  if (transparent != -1)        /* we have a winner for a transparent
-                                 * index - do like gif2png and swap
-                                 * index 0 and index transparent */
+  if (ia_has_transparent_pixels (pixels, numpixels))
     {
-      png_color palette[256];
-      gint i;
+      transparent = find_unused_ia_colour (pixels, numpixels, &colors);
 
-      png_set_tRNS (pp, info, (png_bytep) trans, 1, NULL);
-
-      /* Transform all pixels with a value = transparent to
-       * 0 and vice versa to compensate for re-ordering in palette
-       * due to png_set_tRNS() */
-
-      remap[0] = transparent;
-      remap[transparent] = 0;
-
-      /* Copy from index 0 to index transparent - 1 to index 1 to
-       * transparent of after, then from transparent+1 to colors-1
-       * unchanged, and finally from index transparent to index 0. */
-
-      for (i = 0; i < colors; i++)
+      if (transparent != -1)        /* we have a winner for a transparent
+                                     * index - do like gif2png and swap
+                                     * index 0 and index transparent */
         {
-          palette[i].red = before[3 * remap[i]];
-          palette[i].green = before[3 * remap[i] + 1];
-          palette[i].blue = before[3 * remap[i] + 2];
-        }
+          png_color palette[256];
+          gint i;
 
-      png_set_PLTE (pp, info, palette, colors);
+          png_set_tRNS (pp, info, (png_bytep) trans, 1, NULL);
+
+          /* Transform all pixels with a value = transparent to
+           * 0 and vice versa to compensate for re-ordering in palette
+           * due to png_set_tRNS() */
+
+          remap[0] = transparent;
+          remap[transparent] = 0;
+
+          /* Copy from index 0 to index transparent - 1 to index 1 to
+           * transparent of after, then from transparent+1 to colors-1
+           * unchanged, and finally from index transparent to index 0. */
+
+          for (i = 0; i < colors; i++)
+            {
+              palette[i].red = before[3 * remap[i]];
+              palette[i].green = before[3 * remap[i] + 1];
+              palette[i].blue = before[3 * remap[i] + 2];
+            }
+
+          png_set_PLTE (pp, info, palette, colors);
+        }
+      else
+        {
+          /* Inform the user that we couldn't losslessly save the
+           * transparency & just use the full palette */
+          g_message (_("Couldn't losslessly save transparency, "
+                       "saving opacity instead."));
+          png_set_PLTE (pp, info, (png_colorp) before, colors);
+        }
     }
   else
-    {
-      /* Inform the user that we couldn't losslessly save the
-       * transparency & just use the full palette */
-      g_message (_("Couldn't losslessly save transparency, "
-                   "saving opacity instead."));
-      png_set_PLTE (pp, info, (png_colorp) before, colors);
-    }
+    png_set_PLTE (pp, info, (png_colorp) before, colors);
+
 #else
   info->valid |= PNG_INFO_PLTE;
   info->palette = (png_colorp) before;
@@ -1551,13 +1577,12 @@ respin_cmap (png_structp pp,
 
 }
 
-static gint
+static gboolean
 save_dialog (gint32    image_ID,
              gboolean  alpha)
 {
   PngSaveGui    pg;
   GtkWidget    *dlg;
-  GtkWidget    *frame;
   GtkWidget    *table;
   GtkWidget    *toggle;
   GtkObject    *scale;
@@ -1567,8 +1592,8 @@ save_dialog (gint32    image_ID,
                          NULL, 0,
                          gimp_standard_help_func, "file-png-save-defaults",
 
-			 _("_Load Defaults"), RESPONSE_LOAD_DEFAULTS,
-			 _("_Save Defaults"), RESPONSE_SAVE_DEFAULTS,
+			 _("_Load defaults"), RESPONSE_LOAD_DEFAULTS,
+			 _("_Save defaults"), RESPONSE_SAVE_DEFAULTS,
                          GTK_STOCK_CANCEL,   GTK_RESPONSE_CANCEL,
                          GTK_STOCK_OK,       GTK_RESPONSE_OK,
 
@@ -1581,16 +1606,11 @@ save_dialog (gint32    image_ID,
                     G_CALLBACK (gtk_main_quit),
                     NULL);
 
-  frame = gtk_frame_new (_("Settings"));
-  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, TRUE, TRUE, 0);
-  gtk_widget_show (frame);
-
   table = gtk_table_new (9, 3, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 6);
-  gtk_container_add (GTK_CONTAINER (frame), table);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 12);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), table, TRUE, TRUE, 0);
   gtk_widget_show (table);
 
   pg.interlaced = toggle =
@@ -1605,7 +1625,7 @@ save_dialog (gint32    image_ID,
                     &pngvals.interlaced);
 
   pg.bkgd = toggle =
-    gtk_check_button_new_with_mnemonic (_("Save _Background Color"));
+    gtk_check_button_new_with_mnemonic (_("Save _background color"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 3, 1, 2, GTK_FILL, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), pngvals.bkgd);
   gtk_widget_show (toggle);
@@ -1613,7 +1633,7 @@ save_dialog (gint32    image_ID,
   g_signal_connect (toggle, "toggled",
                     G_CALLBACK (gimp_toggle_button_update), &pngvals.bkgd);
 
-  pg.gama = toggle = gtk_check_button_new_with_mnemonic (_("Save _Gamma"));
+  pg.gama = toggle = gtk_check_button_new_with_mnemonic (_("Save _gamma"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 3, 2, 3, GTK_FILL, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), pngvals.gama);
   gtk_widget_show (toggle);
@@ -1622,7 +1642,7 @@ save_dialog (gint32    image_ID,
                     G_CALLBACK (gimp_toggle_button_update), &pngvals.gama);
 
   pg.offs = toggle =
-    gtk_check_button_new_with_mnemonic (_("Save Layer O_ffset"));
+    gtk_check_button_new_with_mnemonic (_("Save layer o_ffset"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 3, 3, 4, GTK_FILL, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), pngvals.offs);
   gtk_widget_show (toggle);
@@ -1631,7 +1651,7 @@ save_dialog (gint32    image_ID,
                     G_CALLBACK (gimp_toggle_button_update),
                     &pngvals.offs);
 
-  pg.phys = toggle = gtk_check_button_new_with_mnemonic (_("Save _Resolution"));
+  pg.phys = toggle = gtk_check_button_new_with_mnemonic (_("Save _resolution"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 3, 4, 5, GTK_FILL, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), pngvals.phys);
   gtk_widget_show (toggle);
@@ -1640,7 +1660,7 @@ save_dialog (gint32    image_ID,
                     G_CALLBACK (gimp_toggle_button_update), &pngvals.phys);
 
   pg.time = toggle =
-    gtk_check_button_new_with_mnemonic (_("Save Creation _Time"));
+    gtk_check_button_new_with_mnemonic (_("Save creation _time"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 3, 5, 6, GTK_FILL, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), pngvals.time);
   gtk_widget_show (toggle);
@@ -1648,7 +1668,7 @@ save_dialog (gint32    image_ID,
   g_signal_connect (toggle, "toggled",
                     G_CALLBACK (gimp_toggle_button_update), &pngvals.time);
 
-  pg.comment = toggle = gtk_check_button_new_with_mnemonic (_("Save Comme_nt"));
+  pg.comment = toggle = gtk_check_button_new_with_mnemonic (_("Save comme_nt"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 3, 6, 7, GTK_FILL, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), pngvals.comment);
   gtk_widget_show (toggle);
@@ -1663,7 +1683,7 @@ save_dialog (gint32    image_ID,
                     G_CALLBACK (gimp_toggle_button_update), &pngvals.comment);
 
   pg.save_transp_pixels = toggle =
-    gtk_check_button_new_with_mnemonic (_("Save Color _Values From Transparent Pixels"));
+    gtk_check_button_new_with_mnemonic (_("Save color _values from transparent pixels"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 3, 7, 8, GTK_FILL, 0, 0, 0);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle),
                                 alpha && pngvals.save_transp_pixels);
@@ -1676,7 +1696,7 @@ save_dialog (gint32    image_ID,
 
   pg.compression_level = scale =
     gimp_scale_entry_new (GTK_TABLE (table), 0, 8,
-                          _("Co_mpression Level:"),
+                          _("Co_mpression level:"),
                           SCALE_WIDTH, 0,
                           pngvals.compression_level,
                           0.0, 9.0, 1.0, 1.0, 0, TRUE, 0.0, 0.0,
